/**
/*   Copyright (c) 2003by  Marco Welti
/*
/*   This document is  bound by the  QT Public License
/*   (http://www.trolltech.com/licenses/qpl.html).
/*   See License.txt for more information.
/*
/*
/*
/*   ALL RIGHTS RESERVED.  
/* 
/*   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
/*   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
/*   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/*   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
/*   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/*   GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
/*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
/*   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
/*   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
/*   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/* 
/***********************************************************************/

#ifndef ComHelper_H
#define ComHelper_H

#include <string>
#include <sstream>
#include <typeinfo>

#include <comdef.h>

class COMExceptionThrower
{
public:
	COMExceptionThrower( void){}
	~COMExceptionThrower( void){}

	/*COMExceptionThrower( const HRESULT errCode)
	{
		this->operator=(errCode);
	}*/
		
protected:
	// don't allow regular copy constructor call
	COMExceptionThrower( const COMExceptionThrower&);

public:
	inline const COMExceptionThrower& operator=( const HRESULT errCode) const throw( _com_error)
	{
		if ( FAILED( errCode))
			_com_raise_error( errCode);
			
		return *this;
	}
};

inline void DebugBox(const std::string &message)
{
  #ifdef DEBUG
    ::MessageBox(NULL, message.c_str(), "BeSweet AddIn", MB_OK | MB_ICONERROR);
  #endif
}

inline std::string asString(const _com_error &e)
{
  std::stringstream ss;
	ss << "Com-Exception \"" << e.ErrorMessage() << "\"" << std::hex << _com_error::WCodeToHRESULT(e.WCode());
  return ss.str();
}

#define HResult HRESULT STDMETHODCALLTYPE

template<class T>
CComPtr<T> new_instance()
{
  typedef CComObject<T> ComObject;
  ComObject *object;
	ComObject::CreateInstance(&object);

  return CComPtr<T>(object);
}

extern COMExceptionThrower cex_;

#define SANDBOX_BEGIN(class_name, method_name) \
  static const std::string sbox_class_name(class_name);    \
  static const std::string sbox_method_name(method_name);  \
  try {                                                     \


#define SANDBOX_END()                                                             \
  } catch(exception &e) {                                                       \
    ::MessageBox(NULL,                                                          \
        (typeid(e).name() + std::string("\n") + e.what()).c_str(),                \
        (sbox_class_name + "::" + sbox_method_name + "()").c_str(),               \
        MB_OK | MB_ICONERROR);                                           \
                                                                                 \
  } catch(_com_error &e) {                                                       \
    ::MessageBox(NULL,                                                            \
          (typeid(e).name() + std::string("\n") + e.ErrorMessage()).c_str(),         \
          (sbox_class_name + "::" + sbox_method_name + "()").c_str(),             \
          MB_OK | MB_ICONERROR);                                         \
                                                                                \
  } catch(...) {                                                                \
    ::MessageBox(NULL,                                                          \
          "unknown excpetion occured",                                          \
          (sbox_class_name + "::" + sbox_method_name + "()").c_str(),             \
          MB_OK | MB_ICONERROR);                                         \
  }                                                                         \

#endif