/**
/*   Copyright (c) 2003by  Marco Welti
/*
/*   This document is  bound by the  QT Public License
/*   (http://www.trolltech.com/licenses/qpl.html).
/*   See License.txt for more information.
/*
/*
/*
/*   ALL RIGHTS RESERVED.  
/* 
/*   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
/*   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
/*   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/*   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
/*   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/*   GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
/*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
/*   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
/*   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
/*   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/* 
/***********************************************************************/

// ESBDSAddinImpl.h : Declaration of the ESBDSAddinImpl

#ifndef __ESBDSADDIN_H_
#define __ESBDSADDIN_H_

#include "resource.h"       // main symbols
//#import "C:\Program Files\Development\Microsoft Visual Studio\Common\IDE\IDE98\DTE.TLB" raw_interfaces_only, raw_native_types, no_namespace, named_guids 

#include "EventHandler.h"
#include "CommandHandler.h"
#include "BeSweetPreferencesPanel.h"

#include "ESBServer/src/RemoteInterface/COM/IESBServer.h"

/////////////////////////////////////////////////////////////////////////////
// ESBDSAddinImpl
class ATL_NO_VTABLE ESBDSAddinImpl : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<ESBDSAddinImpl, &CLSID_ESBDSAddin>,
	public IDispatchImpl<IESBDSAddin, &IID_IESBDSAddin, &LIBID_ESBDSADDINLib>,
  public IDSAddIn
{
private:
  CComPtr<CommandHandler>	mCommandHandler;
  CComPtr<ApplicationEventHandler> mApplicationEventHandler;
  CComPtr<IApplication> mApplication;
  DWORD mClassRegistrationCookie;
  DWORD mCookie;
  
  //bool  mIsEnabled;
  //std::string mWorkspace;
  //std::string mOptions;

  BeSweetPreferencesPanel mProperties;

public:
	ESBDSAddinImpl();

public:
  //IDSAddIn
	HResult OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,	long dwCookie, VARIANT_BOOL* OnConnection);
	HResult OnDisconnection(VARIANT_BOOL bLastTime);

  // IESBDSAddInImpl
  HResult gotoLine(BSTR filename, long line);

  CComPtr<IApplication>& getApplication()
  { return mApplication; }

  CComPtr<ApplicationEventHandler>& getEventHandler()
  { return mApplicationEventHandler; }
  
  bool isEnabled() { return mProperties.enabled(); }
  void enable(bool e);
  
  void modifyProperties();
  void loadWorkspace(const std::string &, const std::string &);
  void closeWorkspace();
  void reopenWorkspace();
  void documentChanged(const std::string &);
  void documentActivated(const std::string &);

  void showBrowser(const std::string &browsers, const std::string arg);
  void trace(const std::string &);

private:
  BeSweetPreferencesPanel& getProperties() { return mProperties; }
  bool isWorkspaceValid() { return !getProperties().getWorkspace().empty(); }

  void setupComStuff();
  void setupAddIn(bool);
  CComPtr<IESBServer> getESBServer();
  CComPtr<IESBServerManager> getServerManager();
  bool handleDisabledState();
  
public:
  //the atl crap goes here
  DECLARE_REGISTRY_RESOURCEID(IDR_ESBDSADDIN)

  DECLARE_CLASSFACTORY_SINGLETON(ESBDSAddinImpl);

  BEGIN_COM_MAP(ESBDSAddinImpl)
	  COM_INTERFACE_ENTRY(IESBDSAddin)
    COM_INTERFACE_ENTRY(IDSAddIn)
	  COM_INTERFACE_ENTRY(IDispatch)
  END_COM_MAP()

};

#endif //__ESBDSADDIN_H_
