/**
/*   Copyright (c) 2003by  Marco Welti
/*
/*   This document is  bound by the  QT Public License
/*   (http://www.trolltech.com/licenses/qpl.html).
/*   See License.txt for more information.
/*
/*
/*
/*   ALL RIGHTS RESERVED.  
/* 
/*   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
/*   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
/*   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/*   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
/*   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/*   GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
/*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
/*   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
/*   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
/*   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/* 
/***********************************************************************/

#include "stdafx.h"
#include "resource.h"
#include "ESBDSAddin.h"

#include "EventHandler.h"
#include "DSAddin.h"

#include <string>
#include <vector>
#include <iterator>
#include <sstream>

using namespace std;

#define function sbox_class_name+"::"+sbox_method_name

HRESULT ApplicationEventHandler::DocumentSave(IDispatch* theDocument)
{
  SANDBOX_BEGIN("ApplicationEventHandler", "DocumentSave")
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  getAddIn()->trace(function);

  CComQIPtr<ITextDocument> active(theDocument);

  BSTR name;
  cex_ = active->get_FullName(&name);

  getAddIn()->documentChanged(string(bstr_t(name)));

  SANDBOX_END()
    
  return S_OK;
}


HRESULT ApplicationEventHandler::WindowActivate(IDispatch* theWindow)
{
  SANDBOX_BEGIN("ApplicationEventHandler", "WindowActivate")
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
  getAddIn()->trace(function);

  CComPtr<IDispatch> tmp;

  CComQIPtr<ITextWindow> pWindow(theWindow);


  cex_ = pWindow->get_Application(&tmp);
  CComQIPtr<IApplication> pApplication(tmp);

  tmp = 0;
  cex_ = pApplication->get_ActiveDocument(&tmp);
  CComQIPtr<ITextDocument> active(tmp);
  
  BSTR name;
  cex_ = active->get_FullName(&name);

  getAddIn()->documentActivated(string(bstr_t(name)));

  SANDBOX_END()
    
  return S_OK;
}


vector<string> findFile(string file) 
{
  vector<string> result;
  
  WIN32_FIND_DATA rWFD;
  HANDLE hFF= FindFirstFile( file.c_str(), &rWFD );
  if ( hFF == INVALID_HANDLE_VALUE ) {
    return result;
  }
  
  do {
    result.push_back(rWFD.cFileName);
  } while ( FindNextFile( hFF, &rWFD) );
  
  FindClose( hFF );
  return result;
}

HRESULT ApplicationEventHandler::WorkspaceOpen()
{
  static const string f = "ApplicationEventHandler::WorkspaceOpen";
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  getAddIn()->trace(f);

  BSTR cwd;
  getAddIn()->getApplication()->get_CurrentDirectory(&cwd);
  
  vector<string> wspaces(findFile("*.dsw"));
  if(wspaces.size() > 1) {
    stringstream ss;
    ss << "found several .dsw files in directory " << string(bstr_t(cwd)) << endl << endl;
    copy(wspaces.begin(), wspaces.end(), ostream_iterator<string>(ss, "\n"));
    ss << endl << endl << "will open " << wspaces.front();
    ::MessageBox(0, ss.str().c_str(), "Be Sweet", MB_OK);
    
  } else if(wspaces.empty()) {
    ::MessageBox(0, "no workspace file (*.dsw) found in directory\n" + bstr_t(cwd), "Be Sweet Addin", MB_OK | MB_ICONERROR | MB_TOPMOST);
    static char filter[] = "VC6 Workspaces (*.dsw)|*.dsw|VC7 Workspaces (*.sln)|*.sln|All Files (*.*)|*.*||";
    CFileDialog dlg(true, NULL, bstr_t(cwd) + "\\*.dsw", 0, filter);
    
    //dlg.m_ofn.lpstrInitialDir = CString(cwd);
    //::MessageBox(0, "inital dir is " + CString(dlg.m_ofn.lpstrInitialDir), "caption", MB_OK);
    
    if(dlg.DoModal() != IDOK) {
      return E_FAIL;
    }
    wspaces.push_back(dlg.GetPathName ().GetBuffer(0));
  }

  return WorkspaceOpen(string(bstr_t(cwd) + "\\" + wspaces.front().c_str()));
}

HRESULT ApplicationEventHandler::WorkspaceOpen(const std::string &workspace)
{
  SANDBOX_BEGIN("ApplicationEventHandler", "WorkspaceOpen2")
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  getAddIn()->trace(function);

  CComPtr<IDispatch> tmp;
  cex_ = getAddIn()->getApplication()->get_Projects(&tmp.p);
  CComQIPtr<IProjects> projects(tmp);

  long cnt;
  cex_ = projects->get_Count(&cnt);

  string projectNames;
  for(int i = 1; i <= cnt; i++) {
    CComPtr<IGenericProject> project;
    cex_ = projects->Item(CComVariant(i), &project.p);

    CComBSTR name;
    project->get_FullName(&name);
    projectNames += bstr_t(name) + (i == cnt ? "" : ";");
  }
  
  getAddIn()->loadWorkspace(workspace, projectNames);
  
  SANDBOX_END()
    
  return S_OK;
}

HRESULT ApplicationEventHandler::WorkspaceClose()
{
  SANDBOX_BEGIN("ApplicationEventHandler", "WorkspaceClose")

	AFX_MANAGE_STATE(AfxGetStaticModuleState());
  getAddIn()->trace(function);

	getAddIn()->closeWorkspace();

  SANDBOX_END()
    
  return S_OK;
}


// Debugger event

//HRESULT CCommands::XDebuggerEvents::BreakpointHit(IDispatch* pBreakpoint)
//{
//	//AFX_MANAGE_STATE(AfxGetStaticModuleState());
//	return S_OK;

/*
BOOL RecurseFind( LPCSTR szDir, LPCSTR szExt ) 
{
  CString sDir= szDir; 
  CString sWildFile; 
  sWildFile.Format( "%s\\*.*", szDir );
  
  WIN32_FIND_DATA rWFD;
  HANDLE hFF= FindFirstFile( sWildFile, &rWFD );
  if ( hFF == INVALID_HANDLE_VALUE ) {
    return( FALSE );
  }
  do {
    CString sFound= rWFD.cFileName;
    CString sFile;      sFile.Format("%s\\%s", szDir, sFound );
    
    if ( sFound == "." )  continue;
    if ( sFound == "..")  continue;
    if ( (rWFD.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0) {
      RecurseFind( sFile, szExt );
    }
    else {
      sFound.MakeUpper();
      if ( sFound.Right( strlen(szExt) ) == szExt ) {
        // found a matching file!  Save it or something...
        gasAllMatchingFiles.Add( sFile );
      }
    }
  } while ( FindNextFile( hFF, &rWFD) );
  FindClose( hFF );
  return( TRUE );
}
*/