/**
/*   Copyright (c) 2003by  Marco Welti
/*
/*   This document is  bound by the  QT Public License
/*   (http://www.trolltech.com/licenses/qpl.html).
/*   See License.txt for more information.
/*
/*
/*
/*   ALL RIGHTS RESERVED.  
/* 
/*   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
/*   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
/*   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/*   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
/*   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/*   GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
/*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
/*   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
/*   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
/*   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/* 
/***********************************************************************/

#ifndef OutlineBrowser_H
#define OutlineBrowser_H

#include <Utilities/smart_ptr.h>
#include <Model/FileOutlineLoader.h>
#include <Controller/Browsers/SelectionListener.h>

#include <Renderer/OutlineRenderer.h>

#include "Editor.h"

class ESBServer;
class MouseEvent;


struct OutlineFilter : public MetaObjectVisitorAdaptor
{
	OutlineFilter()
	: sortByName(false), publicMembersOnly(false), noStaticMembers(false), noFields(false){}
	
	bool sortByName;
	bool publicMembersOnly;
	bool noStaticMembers;
	bool noFields;

  std::list<smart_ptr<MetaObject> > apply(const std::list<smart_ptr<MetaObject> > &);

  bool isFiltered(const MetaObject&);
  bool isFiltered(const Field &);
  bool isFiltered(const Method&);

private:
  bool mIsFiltered;
  bool mChildrenVisible;
  bool isFiltered() const { return mIsFiltered; }
  std::list<smart_ptr<MetaObject> > selectTags(const std::list<smart_ptr<MetaObject> >&, size_t type) const;

  virtual void visitTag(MetaObject*);
  virtual void visitNamespace(Namespace*);
  virtual void visitField(Field*);
  virtual void visitMethod(Method*);
};


class OutlineBrowser
{
protected:
	smart_ptr<OutlineRenderer> mRenderer;
	FileOutlineLoader::TagList mModel;
	OutlineFilter mFilter;
  std::string mOutlineName;
  MetaObject *mSelection;
  ESBServer *mESBServer;

  std::list<smart_ptr<SelectionListener> > mSelectionListeners;

public:
	OutlineBrowser();
	~OutlineBrowser();
  
  void addSelectionListener(const smart_ptr<SelectionListener>& listener)
  { mSelectionListeners.push_back(listener); }

  void removeSelectionListener(const smart_ptr<SelectionListener>& listener)
  { mSelectionListeners.remove_if(std::bind2nd(std::equal_to<smart_ptr<SelectionListener> >(), listener)); }

  const std::string& getModelName() { return mOutlineName; }
  
	FileOutlineLoader::TagList& getModel() { return mModel; }
  void setModel(const FileOutlineLoader::TagList &model, const std::string &fn = "");
  
	OutlineRenderer& getRenderer() { return *mRenderer; }
  void setRenderer(const smart_ptr<OutlineRenderer> &r) { mRenderer = r; mRenderer->registerListener(this); };

  ESBServer* getESBServer() { if(!mESBServer) {throw NullPointerException("getESBServer"); }; return mESBServer; }
  void setESBServer(ESBServer *srv) { mESBServer = srv; }

  void show();
  void moveToForeground();

  void gotoHeader();
  void gotoSource();
  void gotoHierarchy();
  void inspect();

	void tagSelectionChanged(MetaObject*,  const MouseEvent &);
  void tagSelectionChanged(const std::list<MetaObject*>&);
	
  void sortOrderChanged(bool);
	void publicMemberFilterChanged(bool);
	void staticMemberFilterChanged(bool);
	void fieldFilterChanged(bool);

  std::list<smart_ptr<MetaObject> > getChildrenOf(MetaObject*);


protected:
	OutlineFilter& getFilter() { return mFilter; }
  //void gotoFile(const TagLocation&);

  void setSelection(MetaObject *s);
  MetaObject* getSelection() { return mSelection; }

  std::string makeCaption(const std::string &);

};


#endif
