/**
/*   Copyright (c) 2003by  Marco Welti
/*
/*   This document is  bound by the  QT Public License
/*   (http://www.trolltech.com/licenses/qpl.html).
/*   See License.txt for more information.
/*
/*
/*
/*   ALL RIGHTS RESERVED.  
/* 
/*   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
/*   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
/*   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
/*   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
/*   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
/*   GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
/*   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
/*   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
/*   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
/*   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/* 
/***********************************************************************/

#include "stdafx.h"
#include "resource.h"       // main symbols

#include "ESBServerComModule.h"
extern ESBServerComModule _Module;
#include <atlcom.h>

#include "IESBServer.h"
#include "IESBServerManagerImpl.h"
#include <RemoteInterface/COM/IESBServerImpl.h>
#include <RemoteInterface/COM/VSEditor.h>
#include <Controller/ESBServerManager.h>

#include "ComHelper.h"

COMExceptionThrower cex_;

using namespace std;

/////////////////////////////////////////////////////////////////////////////
// IESBServerManagerImpl

using namespace std;

STDMETHODIMP IESBServerManagerImpl::newWorkspaceServer(BSTR n, IESBDSAddin *vs, IESBServer **srv)
{
  try {
    //AFX_MANAGE_STATE(AfxGetStaticModuleState())
    //message_box("IESBServerManagerImpl::newWorkspaceServer");
    string name(asString(n));
    ContainerType::iterator server = getServer(n);
    if(server != mWorkspaceServers.end()) {
      throw runtime_error("workspace <" + name + "> already exists !");
      //mWorkspaceServers.erase(server);
    }

    //message_box(("creating workspace <" + name + ">").c_str());
    ESBServer *real = &ESBServerManager::instance().newWorkspaceServer(name);
    real->setEditorBroker(make_smart_ptr(new VisualStudioEditorBroker(CComPtr<IESBDSAddin>(vs))));

    CComQIPtr<IESBServer> comServer;
    cex_ = comServer.CoCreateInstance(CLSID_ESBServerCoClass);
    dynamic_cast<IESBServerImpl*>(comServer.p)->setServer(real);
    server = mWorkspaceServers.insert(server, make_pair(name, comServer));

    *srv = server->second;
    (*srv)->AddRef();
    return S_OK;
    
  } catch(std::exception &e) {
    message_box("IESBServerManagerImpl::newWorkspaceServer(): " + string(e.what()));
  }
	return E_FAIL;
}

STDMETHODIMP IESBServerManagerImpl::getWorkspaceServer(BSTR n, IESBServer **srv)
{
  try {
    ContainerType::iterator server = getServer(n);
    if(server != mWorkspaceServers.end()) {
      //message_box((string("found workspace <") + asString(n) + ">").c_str());
      *srv = server->second;
      (*srv)->AddRef();
      return S_OK;
    }
    
    //message_box((string("workspace <") + asString(n) + "> not found").c_str());
  } catch(std::exception &e) {
    message_box(string("IESBServerManagerImpl::getWorkspaceServer(): ") + e.what());
  }
  return E_FAIL;
}

STDMETHODIMP IESBServerManagerImpl::removeWorkspaceServer(BSTR n)
{
  try {
    //message_box((string("removing workspace <") + asString(n) + ">").c_str());
    mWorkspaceServers.erase(asString(n));
    ESBServerManager::instance().removeWorkspaceServer(asString(n));
    return S_OK;
    
  } catch(std::exception &e) {
    message_box(string("IESBServerManagerImpl::removeWorkspaceServer(): ") + e.what());
  }
  return E_FAIL;
}

IESBServerManagerImpl::ContainerType::iterator IESBServerManagerImpl::getServer(const BSTR &n)
{
  return mWorkspaceServers.find(asString(n));
}
